
import streamlit as st

from defermi.gui.main import pages_dict
from defermi.gui.inputs import upload_file, load_file
from defermi.gui.info import title
from defermi.gui.utils import init_state_variable, insert_space, load_session_from_preset, reset_session, widget_with_updating_state





def main():
    st.set_page_config(layout="wide")
    cols = st.columns(3)
    with cols[1]:
    # Inject CSS that removes the border radius from the *next* st.image()
        st.markdown("""
        <style>
        /* Select the most recently created stImage (the last one) */
        [data-testid="stImage"] img {
            border-radius: 0 !important;
        }
        </style>
        """, unsafe_allow_html=True)
        st.image(title, width=300)

    st.divider()
    insert_space(100)


    st.markdown(
        """
        <div style='text-align: center; font-size: 32px; font-weight: bold; letter-spacing: 1px;'>
            Welcome to the defermi UI!
            </span>
            <span style='font-weight: normal; font-size: 28px; margin-left: 10px;'>
                — A tool to analyse point-defects
            </span>
        </div>
        """,
        unsafe_allow_html=True
    )

    st.markdown(
        """
        <div style='text-align: center; font-size: 24px; color: #333333; margin-top: 10px;'>
            Load a file or a preset to get started
        </div>
        """,
        unsafe_allow_html=True
    )

    insert_space(100)
    cols = st.columns([0.7,0.3])
    with cols[0]:
        st.markdown('## 📄 Presets')
        init_state_variable('presets',value=None)
        options = ['Vacancies','Vacancies + Interstitial']

        def change_preset():
            widget_presets = st.session_state['widget_presets']
            st.session_state.clear()
            st.session_state['presets'] = widget_presets
            return


        st.markdown("""
            <style>
                .stMultiSelect [data-baseweb=select] span{
                    max-width: 1000px;
                    font-size: 1.0rem;
                }
            </style>
            """, unsafe_allow_html=True)
        try:
            presets = st.multiselect(
                                    label='presets',
                                    options=options,
                                    default=st.session_state['presets'],
                                    key='widget_presets',
                                    label_visibility='collapsed',
                                    max_selections=1,
                                    on_change=change_preset)
        except st.errors.StreamlitDuplicateElementKey: # catch duplicate key error on start-up, re-running the app make it disappear
            st.rerun()
        
        st.session_state['presets'] = presets

        preset = presets[0] if presets else None
        if preset and not st.session_state['session_loaded']:

            if preset == 'Vacancies':
                filename = 'vacancies.defermi'
                load_session_from_preset(filename=filename)

            if preset == 'Vacancies + Interstitial':
                filename = 'vacancies_interstitials.defermi'
                load_session_from_preset(filename=filename)

            st.session_state['session_loaded'] = True
            st.switch_page(pages_dict['overview'])  
        
            
        
if __name__ == '__main__':
    main()




import streamlit as st
import pandas as pd
import time

from defermi import DefectsAnalysis
from defermi.gui.info import dataset_info, names_info
from defermi.gui.utils import init_state_variable, store_edited_df



def main():
    st.title("Data")
    st.set_page_config(layout="wide")

    st.write('')
    cols = st.columns([0.1,0.7,0.1])
    with cols[0]:
        def reset_dataframes():
            st.session_state.pop('complete_dataframe',None)
            return 
        st.button('Reset',key='widget_reset_da',on_click=reset_dataframes)

    with cols[1]:
        if st.session_state.da:
            csv_str = st.session_state.da.to_dataframe(include_data=False,include_structures=False).to_csv(index=False)
            filename = st.session_state['session_name'] + '_dataset.csv'
            st.download_button(
                label="💾 Save csv",
                data=csv_str,
                file_name=filename,
                mime="test/csv")   
    with cols[2]:
        with st.popover(label='ℹ️',help='Info',type='tertiary'):
            pass
            st.write(dataset_info)

    data = st.session_state['complete_dataframe']
    edited_df = st.data_editor(
                    data, 
                    column_config={
                        'Include':st.column_config.CheckboxColumn(default=True),
                        'name':st.column_config.TextColumn(default='Vac_O',help=names_info),
                        'charge':st.column_config.NumberColumn(default=None),
                        'multiplicity':st.column_config.NumberColumn(default=None),
                        'energy_diff':st.column_config.NumberColumn(default=None),
                        'bulk_volume':st.column_config.NumberColumn(default=None),
                        },
                    hide_index=True,
                    num_rows='dynamic',
                    height='stretch',
                    key='widget_complete_dataframe',
                    on_change=store_edited_df,  # prevent double-clicking problem
                    args=['complete_dataframe'])
    

    st.session_state['complete_dataframe'] = edited_df
    st.session_state['dataframe'] = edited_df

    st.session_state.pop('formation_energies_figure',None)


if __name__ == '__main__':
    main()



import numpy as np
import uuid
import streamlit as st
import matplotlib.pyplot as plt

from pymatgen.core.composition import Composition

from defermi.plotter import plot_pO2_vs_concentrations, plot_pO2_vs_fermi_level
from defermi.gui.info import precursors_info, oxygen_ref_info, cache_info, concentrations_mode_info, brouwer_diagram_info
from defermi.gui.utils import init_state_variable, download_plot, _get_axis_limits_with_widgets, _filter_concentrations


def oxygen_ref():
    init_state_variable('oxygen_ref',value=-4.95)
    subcols = st.columns([0.2,0.8])
    with subcols[0]:
        oxygen_ref = st.number_input('**μO (0K, p0) [eV]**', value=st.session_state['oxygen_ref'], step=0.5, key='widget_oxygen_ref',label_visibility='visible')
        st.session_state['oxygen_ref'] = oxygen_ref
    with subcols[1]:
        with st.popover(label='ℹ️',help='Info',type='tertiary'):
            st.write(oxygen_ref_info)


def precursors():
    if st.session_state.da:
        da = st.session_state.da
        cols = st.columns([0.9,0.1])
        with cols[0]:
            st.markdown("##### Precursors")
        with cols[1]:
            with st.popover(label='ℹ️',help='Info',type='tertiary'):
                st.write(precursors_info)

        init_state_variable('precursor_entries',value=[]) 
        
        cols = st.columns([0.1, 0.25, 0.25, 0.3, 0.1])
        with cols[0]:
            add_precursors = st.button("➕",key="widget_add_precursor")
            if add_precursors:
                # Generate a unique ID for this entry
                entry_id = str(uuid.uuid4())
                st.session_state.precursor_entries.append({
                    "id": entry_id,
                    "composition": "",
                    "energy": 0.0
                })

        def remove_precursor_entry(entry_id):
            for idx,entry in enumerate(st.session_state['precursor_entries']):
                if entry['id'] == entry_id:
                    del st.session_state['precursor_entries'][idx]

        init_state_variable('precursor_DB_warning',value=None)
        def set_precursor_energy_from_DB(entry):
            if entry['composition']:
                energy_pfu = pull_stable_energy_pfu_from_composition(composition=entry['composition'])
                st.session_state[f"widget_energy_{entry['id']}"] = energy_pfu
            else:
                st.session_state['precursor_DB_warning'] = 'Enter composition to pull energy from MP database'
            return 


        for entry in st.session_state['precursor_entries']:
            with cols[1]:
                entry["composition"] = st.text_input("Composition", value=entry["composition"], key=f"widget_comp_{entry['id']}")
            with cols[2]:
                widget_key = f"widget_energy_{entry['id']}"
                if widget_key not in st.session_state:
                    st.session_state[widget_key] = entry['energy']
                energy = st.number_input("Energy p.f.u (eV)", step=1.0, key=f"widget_energy_{entry['id']}")
                entry["energy"] = energy
            with cols[3]:
                st.write('')
                st.button('🗄️ Materials Project DB',on_click=set_precursor_energy_from_DB,args=[entry],key=f"widget_pull_{entry['id']}")
            if st.session_state['precursor_DB_warning']:
                st.error(st.session_state['precursor_DB_warning'])
                st.session_state['precursor_DB_warning'] = None
            with cols[4]:
                st.write('')
                st.button("🗑️", on_click=remove_precursor_entry, args=[entry['id']], key=f"widget_del_{entry['id']}")

        st.session_state['precursors'] = {
                            entry["composition"]: entry["energy"] 
                            for entry in st.session_state.precursor_entries
                            if entry["composition"]}



def filter_entries_with_missing_elements():
    """
    Remove defect entries with elements missing from precursors from brouwer diagram dataset.
    """
    if "precursors" in st.session_state and st.session_state.da:
        precursors = st.session_state['precursors']
        da = st.session_state.da
        # remove defect entries with missing precursors from brouwer diagram dataset
        elements_in_precursors = set()
        if precursors:
            for comp in precursors:
                if comp:
                    for element in Composition(comp).elements:
                        elements_in_precursors.add(element.symbol)

        filter_elements = set()
        missing_elements = set()
        for el in da.elements:
            if el in elements_in_precursors:
                filter_elements.add(el)
            else:
                missing_elements.add(el)

        cols = st.columns(5)
        for idx,el in enumerate(missing_elements):
            ncolumns = 5
            col_idx = idx%ncolumns
            with cols[col_idx]:
                st.warning(f'{el} missing from precursors')

        if filter_elements:
            try:
                brouwer_da = da.filter_entries(elements=filter_elements)
            except AttributeError:
                st.warning('No entries for Brouwer diagram calculation')
                brouwer_da = None
        else:
            brouwer_da = None       
        
        st.session_state['brouwer_da'] = brouwer_da
        st.session_state['quenched_species_brouwer'] = []
        if brouwer_da:
            if st.session_state['quenched_species']:
                if st.session_state['quench_elements']:
                    for entry in brouwer_da.entries:
                        for df in entry.defect:
                            if df.type == 'Vacancy':
                                if df.name in st.session_state['quenched_species']:
                                    st.session_state['quenched_species_brouwer'].append(df.name)
                            else:
                                if df.specie in st.session_state['quenched_species']:
                                    st.session_state['quenched_species_brouwer'].append(df.specie)

                else:
                    for species in st.session_state['quenched_species']:
                        if species in brouwer_da.names:
                            st.session_state['quenched_species_brouwer'].append(species)
    else:
        st.session_state.pop('brouwer_diagram_figure',None)
        st.session_state.pop('fermi_level_brouwer_figure',None)
        



def pull_stable_energy_pfu_from_composition(composition,thermo_types=['GGA_GGA+U'],**kwargs):
    import base64
    from defermi.tools.materials_project import MPDatabase

    API_KEY = base64.b64decode('Q0FVMk8yODZmRUI2cGJWOUszTU9qblFFUFJkZW9BQXg=').decode()
    energy_pfu = MPDatabase(API_KEY=API_KEY).get_stable_energy_pfu_from_composition(
                                                                                composition=composition,
                                                                                thermo_types=thermo_types,
                                                                                **kwargs)
    return energy_pfu


@st.cache_data
def compute_brouwer_diagram(_brouwer_da):
    _brouwer_da.plot_brouwer_diagram(
                            bulk_dos=st.session_state['dos'],
                            temperature=st.session_state['temperature'],
                            quench_temperature=st.session_state['quench_temperature'],
                            quenched_species=st.session_state['quenched_species_brouwer'],
                            quench_elements = st.session_state['quench_elements'],
                            precursors=st.session_state['precursors'],
                            oxygen_ref=st.session_state['oxygen_ref'],
                            pressure_range=st.session_state['pressure_range'],
                            external_defects=st.session_state['external_defects'],
                            npoints=st.session_state['npoints']
                        )
    return _brouwer_da.thermodata


def get_pO2_vs_fermi_level_figure(xlim,ylim):
    if st.session_state['brouwer_thermodata']:    
        figsize = (6,6)
        da = st.session_state.da
        thermodata = st.session_state.brouwer_thermodata

        fig = plot_pO2_vs_fermi_level(
                partial_pressures=thermodata.partial_pressures,
                fermi_levels=thermodata.fermi_levels,
                band_gap=da.band_gap,
                figsize=figsize,
                fontsize=st.session_state['fontsize'],
                xlim=xlim,
                ylim=ylim
        )
        fig.grid()
        fig.title('Brouwer diagram')
        fig.xlabel(plt.gca().get_xlabel(), fontsize=st.session_state['label_size'])
        fig.ylabel(plt.gca().get_ylabel(), fontsize=st.session_state['label_size'])
        ax = fig.gca()
        fig = ax.get_figure()
        fig.patch.set_alpha(st.session_state['alpha'])
        ax.patch.set_alpha(st.session_state['alpha'])
        return fig



def main():

    st.set_page_config(layout="wide")
    cols = st.columns([0.4,0.6])
    with cols[0]:
        st.title('Brouwer Diagram')
    with cols[1]:
        with st.popover(label='ℹ️',help='Info',type='tertiary'):
            st.write(brouwer_diagram_info)
    if st.session_state.da and 'O' in st.session_state.da.elements:
        st.write('')
        oxygen_ref()
        precursors()
        filter_entries_with_missing_elements()

        cols = st.columns([0.3,0.2,0.2,0.3])
        with cols[1]:
            if st.button('Compute',key='widget_clear_cache_brouwer'):
                compute_brouwer_diagram.clear()
        with cols[2]:
            with st.popover(label='ℹ️',help='Info',type='tertiary'):
                st.write(cache_info)

        st.divider()


        if "dos" in st.session_state and "precursors" in st.session_state:
            if st.session_state['precursors']:
                pressure_range = st.session_state['pressure_range']
                brouwer_da = st.session_state['brouwer_da'] 
                if brouwer_da:
                    cols = st.columns([0.7,0.3])
                    with cols[1]:
                        default_xlim = int(np.log10(pressure_range[0])) , int(np.log10(pressure_range[1]))
                        set_xlim, xlim = _get_axis_limits_with_widgets(
                                                                    label='xlim (log)',
                                                                    key='brouwer',
                                                                    default=default_xlim,
                                                                    boundaries=default_xlim) 
                        xlim = (float(10**xlim[0]) , float(10**xlim[1]))
                        xlim = xlim if set_xlim else pressure_range

                        set_ylim, ylim = _get_axis_limits_with_widgets(
                                                                    label='ylim (log)',
                                                                    key='brouwer',
                                                                    default=(-20,25),
                                                                    boundaries=(-50,30))
                        ylim = (float(10**ylim[0]) , float(10**ylim[1]))
                        ylim = ylim if set_ylim else None   

                        brouwer_thermodata = compute_brouwer_diagram(_brouwer_da=brouwer_da) # leading underscore tells streamlit not to hash the argument
                        dc = brouwer_thermodata.defect_concentrations[0]
                        output, names, charges, colors = _filter_concentrations(dc,key='brouwer')

                    with cols[0]:  
                        fig = plot_pO2_vs_concentrations(
                                                    thermodata=brouwer_thermodata,
                                                    output=output,
                                                    figsize=st.session_state['figsize'],
                                                    fontsize=st.session_state['fontsize'],
                                                    xlim=xlim,
                                                    ylim=ylim,
                                                    colors=colors,
                                                    names=names,
                                                    charges=charges)                                           

                        fig.grid()
                        fig.xlabel(plt.gca().get_xlabel(), fontsize=st.session_state['label_size'])
                        fig.ylabel(plt.gca().get_ylabel(), fontsize=st.session_state['label_size'])
                        ax = fig.gca()
                        fig = ax.get_figure()
                        fig.patch.set_alpha(st.session_state['alpha'])
                        ax.patch.set_alpha(st.session_state['alpha'])
                        st.session_state['brouwer_thermodata'] = brouwer_thermodata
                        st.session_state['brouwer_diagram_figure'] = fig
                        st.pyplot(fig, clear_figure=False, width="stretch")

                        fig = get_pO2_vs_fermi_level_figure(xlim,ylim)
                        st.session_state['fermi_level_brouwer_figure'] = fig

                    with cols[1]:
                        with st.popover(label='ℹ️',help='Info',type='tertiary'):
                            st.write(concentrations_mode_info)
                        st.write('')
                        download_plot(fig=fig,filename='brouwer_diagram.pdf')

    elif st.session_state.da and 'O' not in st.session_state.da.elements:
        st.warning('Brouwer analysis concerns only systems containing Oxygen')


if __name__ == '__main__':
    main()


import streamlit as st
import matplotlib.pyplot as plt

from defermi.gui.info import names_info
from defermi.gui.utils import download_plot, _filter_names, _get_axis_limits_with_widgets


def main():

    st.set_page_config(layout="wide")
    st.title("Binding Energies")

    if st.session_state.da:
        da = st.session_state.da
        cols = st.columns([0.7,0.3])
        with cols[1]:
            set_xlim, xlim = _get_axis_limits_with_widgets(
                                                        label='xlim',
                                                        key='binding',
                                                        default=(-0.5,da.band_gap+0.5),
                                                        boundaries=(-3.,da.band_gap+3.)) 
            xlim = xlim if set_xlim else None
            set_ylim, ylim = _get_axis_limits_with_widgets(
                                                        label='ylim',
                                                        key='ctl',
                                                        default=(-0.5,da.band_gap+0.5),
                                                        boundaries=(-3.,da.band_gap+3.))
            ylim = ylim if set_ylim else None

            complex_names = []
            for entry in da.select_entries(types=['DefectComplex']):
                if entry.name not in complex_names:
                    complex_names.append(entry.name)
            names = _filter_names(defect_names=complex_names,key='binding')

            colors = [st.session_state.color_dict[name] for name in names]
            for color in st.session_state.color_sequence:
                if color not in colors:
                    colors.append(color)

        with cols[0]:
            fig = da.plot_binding_energies(
                names=names,
                figsize=st.session_state['figsize'],
                fontsize=st.session_state['fontsize'],
                colors=colors,
                xlim=xlim,
                ylim=ylim)
            fig.grid()
            fig.xlabel(plt.gca().get_xlabel(), fontsize=st.session_state['label_size'])
            fig.ylabel(plt.gca().get_ylabel(), fontsize=st.session_state['label_size'])
            ax = fig.gca()
            fig = ax.get_figure()
            fig.patch.set_alpha(st.session_state['alpha'])
            ax.patch.set_alpha(st.session_state['alpha'])
            st.pyplot(fig, clear_figure=False, width="content")

        with cols[1]:
            with st.popover(label='ℹ️',help='Info',type='tertiary'):
                st.write(names_info)
            st.write('')
            download_plot(fig=fig,filename='binding_energies.pdf')


if __name__ == '__main__':
    main()



import numpy as np
import streamlit as st
import matplotlib.pyplot as plt

from defermi.plotter import plot_variable_species_vs_concentrations, plot_variable_species_vs_fermi_level
from defermi.gui.info import cache_info, concentrations_mode_info, dopant_info
from defermi.gui.utils import init_state_variable, download_plot, _get_axis_limits_with_widgets, _filter_concentrations


def settings():
    if st.session_state.da:
        cols = st.columns([0.9,0.1])
        with cols[1]:
            with st.popover(label='ℹ️',help='Info',type='tertiary'):
                st.write(dopant_info)

        init_state_variable('dopant_type',value='None')
        init_state_variable('conc_range',value=None)
        init_state_variable('dopant',value={})

        da = st.session_state.da
        possible_dopants = ["None","Donor","Acceptor"]
        for entry in da:
            if entry.defect.type == "Substitution":
                el = entry.defect.specie
                if el not in possible_dopants:
                    possible_dopants.append(el)
        possible_dopants.append('custom')

        # def update_dopant_type_index():
        #     st.session_state['dopant_type'] = st.session_state['widget_select_dopant']
        
        if st.session_state['dopant_type'] in possible_dopants:
            st.session_state['dopant_type_index'] = possible_dopants.index(st.session_state['dopant_type'])
        else:
            st.session_state['dopant_type_index'] = 0
        dopant_type = st.radio("Select dopant",options=possible_dopants,index=st.session_state['dopant_type_index'],
                                        horizontal=True, key='widget_select_dopant')#,on_change=update_dopant_type_index)
        st.session_state['dopant_type'] = dopant_type
        

        dopant_type = st.session_state['dopant_type']    
        if dopant_type == "None":
            st.session_state['dopant'] = None
            st.session_state['conc_range'] = None
            st.session_state.pop('doping_diagram_figure',None)
        elif dopant_type == "Donor":
            cols = st.columns(2)
            with cols[0]:
                d = st.session_state['dopant']
                if d and type(d) == dict and 'charge' in d and d['charge'] > 0: 
                    value = d['charge']
                else:
                    value = 1.0 
                charge = st.number_input("Charge", min_value=0.0, value=value, step = 1.0, key="widget_donor_charge")
            with cols[1]:
                def update_conc_range():
                    min_conc, max_conc = st.session_state['widget_conc_range']
                    st.session_state['conc_range'] = ( float(10**min_conc), float(10**max_conc) )
                    return
                if st.session_state['conc_range']:
                    value = int(np.log10(float(st.session_state['conc_range'] [0]))), int(np.log10(float(st.session_state['conc_range'] [1])))
                else:
                    value = (5,18)        
                st.slider(r"Range: log₁₀(concentration (cm⁻³))",min_value=-20,max_value=24,value=value,step=1, 
                                                    key="widget_conc_range",on_change=update_conc_range)  
            st.session_state['dopant'] = {"name":"D","charge":charge}
            st.session_state.pop('previous_names_doping',None)

        elif dopant_type == "Acceptor":
            cols = st.columns(2)
            with cols[0]:
                d = st.session_state['dopant']
                if d and type(d) == dict and 'charge' in d and d['charge'] < 0: 
                    value = d['charge']
                else:
                    value = -1.0 
                charge = st.number_input("Charge", max_value=0.0, value=value, step = 1.0, key="widget_acceptor_charge")
            with cols[1]:
                
                def update_conc_range():
                    min_conc, max_conc = st.session_state['widget_conc_range']
                    st.session_state['conc_range'] = ( float(10**min_conc), float(10**max_conc) )
                    return
                if st.session_state['conc_range']:
                    value = int(np.log10(float(st.session_state['conc_range'] [0]))), int(np.log10(float(st.session_state['conc_range'] [1])))
                else:
                    value = (5,18)         
                st.slider(r"Range: log₁₀(concentration (cm⁻³))",min_value=-20,max_value=24,value=value,step=1, 
                                                    key="widget_conc_range",on_change=update_conc_range) 
            st.session_state['dopant'] = {"name":"A","charge":charge}
            st.session_state.pop('previous_names_doping',None)

        elif dopant_type == "custom":
            cols = st.columns(3)
            d = st.session_state['dopant'] or {}
            with cols[0]:
                value = d['name'] if 'name' in d else 'custom'
                name = st.text_input("Name",value=value, key="widget_name_dopant")          
            with cols[1]:
                if d and type(d) == dict and 'charge' in d:
                    value = d['charge']
                else:
                    value = 0.0
                charge = st.number_input("Charge", value=value, step = 1.0, key="widget_dopant_charge")
            with cols[2]:             
                def update_conc_range():
                    min_conc, max_conc = st.session_state['widget_conc_range']
                    st.session_state['conc_range'] = ( float(10**min_conc), float(10**max_conc) )
                    return
                if st.session_state['conc_range']:
                    value = int(np.log10(float(st.session_state['conc_range'] [0]))), int(np.log10(float(st.session_state['conc_range'] [1])))
                else:
                    value = (5,18)         
                st.slider(r"Range: log₁₀(concentration (cm⁻³))",min_value=-20,max_value=24,value=value,step=1, 
                                                    key="widget_conc_range",on_change=update_conc_range)
            st.session_state['dopant'] = {"name":name,"charge":charge}
            st.session_state.pop('previous_names_doping',None)

        else:
            cols = st.columns(3)
            with cols[2]:
                st.session_state['dopant'] = dopant_type
                def update_conc_range():
                    min_conc, max_conc = st.session_state['widget_conc_range']
                    st.session_state['conc_range'] = ( float(10**min_conc), float(10**max_conc) )
                    return
                if st.session_state['conc_range']:
                    value = int(np.log10(st.session_state['conc_range'] [0])), int(np.log10(st.session_state['conc_range'] [1]))
                else:
                    value = (5,18)      
                st.slider(r"Range: log₁₀(concentration (cm⁻³))",min_value=-20,max_value=24,value=value,step=1, 
                                        key="widget_conc_range",on_change=update_conc_range)

    
        if st.session_state['dopant']:
            if not st.session_state['conc_range']:
                st.session_state['conc_range'] = (1e05,1e18)

            cols = st.columns([0.3,0.2,0.2,0.3])
            with cols[1]:
                if st.button('Compute',key='widget_clear_cache_doping'):
                    compute_doping_diagram.clear()
            with cols[2]:
                with st.popover(label='ℹ️',help='Info',type='tertiary'):
                    st.write(cache_info)



@st.cache_data
def compute_doping_diagram():
    st.session_state['da'].plot_doping_diagram(
            variable_defect_specie=st.session_state['dopant'],
            concentration_range=st.session_state['conc_range'],
            chemical_potentials=st.session_state['chempots'],
            bulk_dos=st.session_state['dos'],
            temperature=st.session_state['temperature'],
            quench_temperature=st.session_state['quench_temperature'],
            quenched_species=st.session_state['quenched_species'],
            external_defects=st.session_state['external_defects'],
            npoints=st.session_state['npoints'],
            )
    return st.session_state['da'].thermodata


def get_doping_vs_fermi_level_figure(xlim,ylim=None):
    if st.session_state['doping_thermodata']:    
        figsize = (6,6)
        da = st.session_state['da']
        thermodata = st.session_state['doping_thermodata']

        if type(st.session_state['dopant']) == dict:
            xlabel = st.session_state['dopant']['name']
        else:
            xlabel = st.session_state['dopant']

        fig = plot_variable_species_vs_fermi_level(
                xlabel = xlabel, 
                variable_concentrations=thermodata.variable_concentrations,
                fermi_levels=thermodata.fermi_levels,
                band_gap=da.band_gap,
                figsize=figsize,
                fontsize=st.session_state['fontsize'],
                xlim=xlim,
                ylim=ylim
        )
        fig.grid()
        fig.title('Doping diagram')
        fig.xlabel(plt.gca().get_xlabel(), fontsize=st.session_state['label_size'])
        fig.ylabel(plt.gca().get_ylabel(), fontsize=st.session_state['label_size'])
        ax = fig.gca()
        fig = ax.get_figure()
        fig.patch.set_alpha(st.session_state['alpha'])
        ax.patch.set_alpha(st.session_state['alpha'])
        return fig


def main():

    st.set_page_config(layout="wide")
    st.title("Doping Diagram")
    settings()
    st.divider()

    if "dos" in st.session_state and "dopant" in st.session_state:
        if st.session_state['conc_range']:
            da = st.session_state.da
            conc_range = st.session_state['conc_range']

            cols = st.columns([0.7,0.3])
            with cols[1]:
                default_xlim = int(np.log10(conc_range[0])) , int(np.log10(conc_range[1]))
                set_xlim, xlim = _get_axis_limits_with_widgets(
                                                            label='xlim (log)',
                                                            key='doping',
                                                            default=default_xlim,
                                                            boundaries=default_xlim) 
                xlim = (float(10**xlim[0]) , float(10**xlim[1]))
                xlim = xlim if set_xlim else conc_range

                set_ylim, ylim = _get_axis_limits_with_widgets(
                                                            label='ylim (log)',
                                                            key='doping',
                                                            default=(-20,25),
                                                            boundaries=(-50,30))
                ylim = (float(10**ylim[0]) , float(10**ylim[1]))
                ylim = ylim if set_ylim else None   

                doping_thermodata = compute_doping_diagram()
                dc = doping_thermodata.defect_concentrations[0]
                output, names, charges, colors = _filter_concentrations(dc,key='doping')

            with cols[0]:
                fig = plot_variable_species_vs_concentrations(
                                                doping_thermodata,
                                                output=output,
                                                figsize=st.session_state['figsize'],
                                                fontsize=st.session_state['fontsize'],
                                                colors=colors,
                                                xlim=xlim,
                                                ylim=ylim,
                                                names=names,
                                                charges=charges
                                                )
                fig.grid()
                fig.xlabel(plt.gca().get_xlabel(), fontsize=st.session_state['label_size'])
                fig.ylabel(plt.gca().get_ylabel(), fontsize=st.session_state['label_size'])
                ax = fig.gca()
                fig = ax.get_figure()
                fig.patch.set_alpha(st.session_state['alpha'])
                ax.patch.set_alpha(st.session_state['alpha'])
                st.session_state['doping_thermodata'] = doping_thermodata
                st.session_state['doping_diagram_figure'] = fig
                st.pyplot(fig, clear_figure=False, width="stretch")

                fig = get_doping_vs_fermi_level_figure(xlim,ylim=None)
                st.session_state['fermi_level_doping_figure'] = fig 

            with cols[1]:
                with st.popover(label='ℹ️',help='Info',type='tertiary'):
                    st.write(concentrations_mode_info)
                st.write('')
                download_plot(fig=fig,filename='doping_diagram.pdf')


if __name__ == '__main__':
    main()



import numpy as np
import uuid
import streamlit as st

from defermi.gui.info import quenching_info, external_defects_info
from defermi.gui.utils import init_state_variable


def thermodynamics():
    if st.session_state.da:
        init_state_variable('temperature',value=1000)
        st.markdown("### Temperature (K)")
        temperature = st.slider("Temperature (K)", min_value=0, max_value=1500, value=st.session_state['temperature'], 
                                step=50, key="widget_temperature",label_visibility='collapsed')
        if temperature == 0:
            temperature = 0.1 # prevent division by zero
        st.session_state['temperature'] = temperature

        quenching()
        external_defects()

        st.divider()


def quenching():
    """
    GUI elements to set defect quenching parameters.
    """
    init_state_variable('enable_quench',value=False)
    init_state_variable('quench_temperature',value=None)
    init_state_variable('quench_mode',value='species')
    init_state_variable('quenched_species',value=None)
    init_state_variable('quench_elements',value=None)

    enable_quench = st.checkbox("Enable quenching", value=st.session_state['enable_quench'], key="widget_enable_quench")
    st.session_state['enable_quench'] = enable_quench
    if enable_quench:
        cols = st.columns([0.45,0.45,0.1])
        with cols[2]:
            with st.popover(label='ℹ️',help='Info',type='tertiary'):
                st.write(quenching_info)
        with cols[0]:
            st.session_state['quench_temperature'] = 300
            quench_temperature = st.slider("Quench Temperature (K)", min_value=0, max_value=1500, 
                                        value=st.session_state['quench_temperature'], step=50, key="widget_quench_temperature")
            st.session_state['quench_temperature'] = quench_temperature
        if st.session_state['quench_temperature'] == 0:
            st.session_state['quench_temperature'] = 0.1 

        with cols[1]:
            index = 0 if st.session_state['quench_mode'] == 'species' else 1
            quench_mode = st.radio("Quenching mode",("species","elements"),horizontal=True,key="widget_quench_mode",index=index)

        if quench_mode == "species":
            species = set()
            for entry in st.session_state.da:
                for df in entry.defect:
                    species.add(df.name)
            default = st.session_state['quenched_species'] or species
            if st.session_state['quenched_species']:
                for name in st.session_state['quenched_species']:
                    if name not in species:
                        default = species
            quenched_species = st.multiselect("Select quenched species",options=species,default=default,key='widget_quenched_species')
            quench_elements = False

        elif quench_mode == "elements":
            species = set()
            for entry in st.session_state['da']:
                if entry.defect.type == 'Vacancy':
                    species.add(entry.defect.name)
                else:
                    for df in entry.defect:
                        species.add(df.specie)
            if st.session_state['quench_elements']:
                default = st.session_state['quenched_species'] or species
            else:
                default = species
            quenched_species = st.multiselect("Select quenched elements",options=species,default=default,key='widget_quenched_elements')
            quench_elements = True
    
        st.session_state['quenched_species'] = quenched_species
        st.session_state['quench_elements'] = quench_elements
    else:
        st.session_state['quenched_species'] = None
        st.session_state['quench_elements'] = False
        st.session_state['quench_temperature'] = None



def external_defects():
    """
    GUI elements to set external defects.
    """
    cols = st.columns([0.9,0.1])
    with cols[0]:
        st.markdown("**External defects**")
    with cols[1]:
        with st.popover(label='ℹ️',help='Info',type='tertiary'):
            st.write(external_defects_info)

    init_state_variable('external_defects_entries',value=[])

    cols = st.columns([0.11, 0.26, 0.26, 0.26, 0.11])
    with cols[0]:
        if st.button("➕",key="widget_add_external_defect"):
            # Generate a unique ID for this entry
            entry_id = str(uuid.uuid4())
            st.session_state['external_defects_entries'].append({
                "id": entry_id,
                "name": "",
                "charge": 0.0,
                "conc":1.0})

    def remove_external_defects_entries(entry_id):
        for idx,entry in enumerate(st.session_state['external_defects_entries']):
            if entry['id'] == entry_id:
                del st.session_state['external_defects_entries'][idx]

    for defect in st.session_state['external_defects_entries']:
        with cols[1]:
            name = st.text_input("Name",value=defect['name'], key=f"widget_name_{defect['id']}")
            defect["name"] = name
        with cols[2]:
            charge = st.number_input("Charge", value=defect['charge'], step=1.0,key=f"widget_charge_{defect['id']}")
            defect["charge"] = charge
        with cols[3]:
            value = int(np.log10(float(defect['conc']))) if defect['conc'] else 0
            conc = st.number_input(r"log₁₀(c (cm⁻³))", value=value, step=1, key=f"widget_conc_{defect['id']}")
            defect["conc"] = 10**conc 
        with cols[4]:
            st.button("🗑️", on_click=remove_external_defects_entries, args=[defect['id']], key=f"widget_del_{defect['id']}")

    st.session_state['external_defects'] = [{
                        'name':e['name'],
                        'charge':e['charge'],
                        'conc':e['conc']
                        } for e in st.session_state.external_defects_entries if e["name"]]
        


import streamlit as st
import matplotlib.pyplot as plt
import matplotlib

from defermi.gui.info import names_info
from defermi.gui.utils import init_state_variable, download_plot, _get_axis_limits_with_widgets, _filter_names


class FormationEnergiesPlotter:

    def __init__(self,defects_analysis):
        self.da = defects_analysis

    def get_figure(self,entries=None,colors=None,xlim=None,ylim=None):
        da = self.da
        if 'chempots' not in st.session_state:
            st.warning('Chemical potentials are not defined')
        fig = da.plot_formation_energies(
            entries=entries,
            chemical_potentials=st.session_state['chempots'],
            figsize=st.session_state['figsize'],
            fontsize=st.session_state['fontsize'],
            colors=colors,
            xlim=xlim,
            ylim=ylim)
        fig.grid()
        fig.xlabel(plt.gca().get_xlabel(), fontsize=st.session_state['label_size'])
        fig.ylabel(plt.gca().get_ylabel(), fontsize=st.session_state['label_size'])
        ax = fig.gca()
        fig = ax.get_figure()
        fig.patch.set_alpha(st.session_state['alpha'])
        ax.patch.set_alpha(st.session_state['alpha'])
        st.session_state['formation_energies_figure'] = fig
        return fig


    def get_axis_limits(self):
        da = self.da
        set_xlim, xlim = _get_axis_limits_with_widgets(
                                                    label='xlim',
                                                    key='eform',
                                                    default=(-0.5,da.band_gap+0.5),
                                                    boundaries=(-3.,da.band_gap+3.)) 
        xlim = xlim if set_xlim else None

        set_ylim, ylim = _get_axis_limits_with_widgets(
                                                    label='ylim',
                                                    key='eform',
                                                    default=(-20.,30.),
                                                    boundaries=(-20.,30.))
        ylim = ylim if set_ylim else None
        return xlim,ylim


    def get_entries_and_colors(self,filter_names=True):
        da = self.da
        defect_names = da.names
        if filter_names:
            names = _filter_names(defect_names=defect_names,key='eform')
        else:
            names = defect_names

        entries = da.select_entries(names=names)
        colors = []
        ordered_names = []
        for entry in entries:
            if entry.name not in ordered_names:
                ordered_names.append(entry.name)      
        colors = [st.session_state.color_dict[name] for name in ordered_names]

        return entries,colors


def main():
    st.set_page_config(layout="wide")
    st.title('Formation Energies')

    if st.session_state.da and 'chempots' in st.session_state:
        da = st.session_state.da
        cols = st.columns([0.7,0.3])
        plotter = FormationEnergiesPlotter(defects_analysis=da)
        with cols[1]:
            xlim,ylim = plotter.get_axis_limits()
            entries,colors = plotter.get_entries_and_colors(da)

        with cols[0]:
            fig = plotter.get_figure(entries,colors,xlim,ylim)
            st.pyplot(fig, clear_figure=False, width="content")

        with cols[1]:
            with st.popover(label='ℹ️',help='Info',type='tertiary'):
                st.write(names_info)
            st.write('')                    
            download_plot(fig=fig,filename='formation_energies.pdf')

if __name__ == '__main__':
    main()





